import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';
import 'package:bird_cryptography/bird_cryptography.dart';

class AnnotatedValue<T> {
  final String name;
  final T value;

  const AnnotatedValue(this.name, this.value);
}

Widget closeButton = $$ >> (context) =>
    FlatButton(
      child: const Text("Close"),
      onPressed: () => Navigator.of(context).pop(),
    );

Widget loadingIndicator(Wave<bool> loading) {
  return $$ >> (context) {
    final isLoading = $(() => loading);
    if (isLoading) {
      return square(28) > const CircularProgressIndicator();
    } else {
      return nothing;
    }
  };
}

Applicator calculate(Wave<bool> loading, void Function() convert) {
  return apply((child) =>
  $$ >> (context) {
    final isLoading = $(() => loading);
    if (!isLoading) {
      return flatButton(convert) > child;
    } else {
      return nothing;
    }
  });
}

Widget encodingWaved(Wave<bool> loading,
    Signal<CryptographicDataEncoding> encoding) {
  return $$ >> (context) {
    final isLoading = $(() => loading);
    if (!isLoading) {
      return encodings($(() => encoding.wave), encoding.add);
    } else {
      return nothing;
    }
  };
}

DropdownButton encodings(CryptographicDataEncoding value,
    void Function(CryptographicDataEncoding) onChanged) =>
    DropdownButton<CryptographicDataEncoding>(
      value: value,
      items: [
        DropdownMenuItem(value: e_base64, child: const Text("Base 64")),
        DropdownMenuItem(value: e_hex, child: const Text("Hex")),
        DropdownMenuItem(value: e_utf8, child: const Text("UTF8")),
        DropdownMenuItem(value: e_ascii, child: const Text("Ascii")),
        DropdownMenuItem(value: e_bytelist, child: const Text("List of bytes")),
        DropdownMenuItem(value: e_intlist, child: const Text("List of integers")),
      ],
      onChanged: onChanged,
    );
