import 'dart:html';

import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';
import 'package:neptune_cryptography_example/pages/ciphers.dart';
import 'package:neptune_cryptography_example/pages/encoding.dart';
import 'package:neptune_cryptography_example/pages/hashes.dart';
import 'package:neptune_cryptography_example/pages/hmacs.dart';
import 'package:neptune_cryptography_example/pages/kdfs.dart';
import 'package:neptune_cryptography_example/pages/systems.dart';

void main() => runApp(MyApp());

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: "Flutter Cryptography Tool",
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: _page(),
    );
  }
}

Widget _page() {
  return $$ >> (context) {
    return tabC(length: 6)
    & fluidByMarginH2(700)
    & scaffold(
        color: Colors.white,
        appBar: AppBar(
          centerTitle: true,
          title: textColor(Colors.black)
              > const Text("Flutter Cryptography Tool"),
          actions: [
            flatButton(() =>
                window.open(
                  "https://github.com/modulovalue/bird_cryptography",
                  "https://github.com/modulovalue/bird_cryptography",
                )) > const Text("on GitHub"),
          ],
          backgroundColor: Colors.white,
          elevation: 0.0,
          bottom: TabBar(
            indicatorSize: TabBarIndicatorSize.label,
            indicatorColor: Colors.black,
            labelColor: Colors.black,
            tabs: height(34.0) * const [
              Tab(text: "Hashes"),
              Tab(text: "Ciphers"),
              Tab(text: "HMACs"),
              Tab(text: "KDFs"),
              Tab(text: "Systems"),
              Tab(text: "Encoding"),
            ],
          ),
        )
    )
        > onTabBarView()
            >> [
              HashesPage(),
              CiphersPage(),
              HMACSPage(),
              KDFSPage(),
              SystemsPage(),
              EncodingPage(),
            ];
  };
}
