import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';
import 'package:neptune_cryptography_example/elements.dart';
import 'package:neptune_cryptography_example/widgets/simple_pw_digest_widget.dart';

class KDFSPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return onListView() >> [
      ...[
        AnnotatedValue("sha256_pbkdf2", (int i, int length) {
          return AnnotatedValue("sha256_pbkdf2/i:$i/length:$length",
              CryptographyKDF.dart.sha256_pbkdf2(i: i, length: length));
        }),
        AnnotatedValue("sha384_pbkdf2", (int i, int length) {
          return AnnotatedValue("sha384_pbkdf2/i:$i/length:$length",
              CryptographyKDF.dart.sha384_pbkdf2(i: i, length: length));
        }),
        AnnotatedValue("sha512_pbkdf2", (int i, int length) {
          return AnnotatedValue("sha512_pbkdf2/i:$i/length:$length",
              CryptographyKDF.dart.sha512_pbkdf2(i: i, length: length));
        }),
      ].map(shaPbkdfTile),
      scryptTile(),
    ];
  }
}

// ignore_for_file: close_sinks
class IAndLengthBloc extends HookBloc {
  final Signal<String> i = HookBloc.disposeSink(Signal("10"));
  final Signal<String> length = HookBloc.disposeSink(Signal("32"));

  Wave<int> i_int;
  Wave<int> length_int;

  IAndLengthBloc() {
    i_int = i.wave.map(int.tryParse);
    length_int = length.wave.map(int.tryParse);
  }
}

Widget shaPbkdfTile(AnnotatedValue<
    AnnotatedValue<KDFCrypto> Function(int i, int length)> builder) {
  return $$ >> (context) {
    return ListTile(
      title: Text(builder.name),
      onTap: () {
        showDialog<void>(
          context: context,
          builder: (_) =>
              AlertDialog(
                title: Text(builder.name),
                content: $$ >> (context) {
                  final iLengthBloc = $$$(() => IAndLengthBloc());
                  final i_int = $(() => iLengthBloc.i_int);
                  final length_int = $(() => iLengthBloc.length_int);

                  return singleChildScrollView() > onColumnMinCenterCenter() >> [
                    TextFormField(
                      initialValue: i_int.toString(),
                      decoration: InputDecoration(
                          labelText: "i",
                          border: const OutlineInputBorder(),
                          errorText: i_int == null ?
                                     "Needs to be an integer" : null
                      ),
                      onChanged: iLengthBloc.i.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    TextFormField(
                      initialValue: length_int.toString(),
                      decoration: InputDecoration(
                        labelText: "length",
                        border: const OutlineInputBorder(),
                        errorText: length_int == null ?
                                   "Needs to be an integer" : null,
                      ),
                      onChanged: iLengthBloc.length.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    if (i_int != null && length_int != null)
                          () {
                        final val = builder.value(i_int, length_int);
                        return FlatButton(
                          child: Text("View ${val.name}"),
                          onPressed: () {
                            simplePwDigestDialog(context, val);
                          },
                        );
                      }(),
                  ];
                },
                actions: [
                  closeButton,
                ],
              ),
        );
      },
    );
  };
}

class ScryptBloc extends HookBloc {
  final Signal<String> n = HookBloc.disposeSink(Signal("8"));
  final Signal<String> r = HookBloc.disposeSink(Signal("8"));
  final Signal<String> p = HookBloc.disposeSink(Signal("8"));
  final Signal<String> length = HookBloc.disposeSink(Signal("32"));

  Wave<int> n_int;
  Wave<int> r_int;
  Wave<int> p_int;
  Wave<int> length_int;

  ScryptBloc() {
    n_int = n.wave.map(int.tryParse);
    r_int = r.wave.map(int.tryParse);
    p_int = p.wave.map(int.tryParse);
    length_int = length.wave.map(int.tryParse);
  }
}

Widget scryptTile() {
  return $$ >> (context) {
    return ListTile(
      title: const Text("scrypt"),
      onTap: () {
        showDialog<void>(
          context: context,
          builder: (_) =>
              AlertDialog(
                title: const Text("scrypt"),
                content: $$ >> (context) {
                  final bloc = $$$(() => ScryptBloc());
                  final n_int = $(() => bloc.n_int);
                  final r_int = $(() => bloc.r_int);
                  final p_int = $(() => bloc.p_int);
                  final length_int = $(() => bloc.length_int);

                  return singleChildScrollView() > onColumnMinCenterCenter() >> [
                    TextFormField(
                      initialValue: n_int.toString(),
                      decoration: InputDecoration(
                          labelText: "n",
                          border: const OutlineInputBorder(),
                          errorText: n_int == null ?
                                     "Needs to be an integer" : null
                      ),
                      onChanged: bloc.n.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    TextFormField(
                      initialValue: r_int.toString(),
                      decoration: InputDecoration(
                          labelText: "r",
                          border: const OutlineInputBorder(),
                          errorText: r_int == null ?
                                     "Needs to be an integer" : null
                      ),
                      onChanged: bloc.r.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    TextFormField(
                      initialValue: p_int.toString(),
                      decoration: InputDecoration(
                          labelText: "p",
                          border: const OutlineInputBorder(),
                          errorText: p_int == null ?
                                     "Needs to be an integer" : null
                      ),
                      onChanged: bloc.p.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    TextFormField(
                      initialValue: length_int.toString(),
                      decoration: InputDecoration(
                        labelText: "length",
                        border: const OutlineInputBorder(),
                        errorText: length_int == null ?
                                   "Needs to be an integer" : null,
                      ),
                      onChanged: bloc.length.add,
                      maxLines: 1,
                    ),
                    verticalSpace(8.0),
                    if (n_int != null && r_int != null && p_int != null
                        && length_int != null)
                          () {
                        return FlatButton(
                          child: const Text("View scrypt"),
                          onPressed: () {
                            simplePwDigestDialog(context, AnnotatedValue(
                                "scrypt/n:$n_int/r:$r_int/p:$p_int/length:$length_int",
                                CryptographyKDF.dart.scrypt(n: n_int,
                                    r: r_int,
                                    p: p_int,
                                    length: length_int)));
                          },
                        );
                      }(),
                  ];
                },
                actions: [
                  closeButton,
                ],
              ),
        );
      },
    );
  };
}