import 'dart:convert';
import 'dart:typed_data';

import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';
import 'package:neptune_cryptography_example/elements.dart';
import 'package:neptune_cryptography_example/widgets/input.dart';

class SystemsPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return onListView() >> [
      ethereumKeystoreTile(),
    ];
  }
}

// ignore_for_file: close_sinks
class _KeystoreBloc extends HookBloc {
  final Signal<String> keystore = HookBloc.disposeSink(Signal(""));
  final Signal<String> pw = HookBloc.disposeSink(Signal(""));
  final Signal<CryptographicDataEncoding> pwEncoding = HookBloc.disposeSink(
      Signal(e_utf8));
}

Widget ethereumKeystoreTile() {
  return $$ >> (context) {
    return ListTile(
      title: const Text("Ethereum Keystore"),
      onTap: () {
        showDialog<void>(
          context: context,
          builder: (_) =>
          $$ >> (context) {
            final bloc = $$$(() => _KeystoreBloc());
            return AlertDialog(
              title: const Text("Ethereum Keystore"),
              content: singleChildScrollView() > onColumnMinCenterCenter() >> [
                const Text("Note: this will take a long time."),
                verticalSpace(12.0),
                signalInputWidget(
                  signal: bloc.keystore,
                  label: just("Keystore"),
                ),
                verticalSpace(12.0),
                encodingWaved(just(false), bloc.pwEncoding),
                signalInputWidget(
                  signal: bloc.pw,
                  label: just("Password"),
                  isConfidential: true,
                ),
                verticalSpace(14.0),
                $$ >> (context) {
                  final a = $(() => bloc.keystore.wave);
                  try {
                    final aa = EthereumKeystoreModel()
                        .from(json.decode(a));
                    return Text(aa.debugToString());
                  } catch (e) {
                    return const Text("");
                  }
                }
              ],
              actions: [
                flatButton(() {
                  try {
                    final a = bloc.keystore.value;
                    final eth = EthereumKeystoreModel().from(json.decode(a));
                    final privKey = eth.decryptOrNull(Uint8List.fromList(
                        bloc.pwEncoding.value.decode(bloc.pw.value)));
                    if (privKey != null) {
                      showDialog<void>(
                        context: context,
                        builder: (context) {
                          return AlertDialog(
                            title: const Text("Keystore unlocked"),
                            content: onColumnMinCenterCenter() >> [
                              const Text("Your private key:"),
                              TextFormField(
                                initialValue: e_hex.encode(privKey),
                              ),
                            ],
                          );
                        },
                      );
                    } else {
                      showDialog<void>(
                        context: context,
                        builder: (context) {
                          return const AlertDialog(
                            title: Text("Keystore could not be unlocked"),
                            content: Text(
                                "Did you enter the correct password?"),
                          );
                        },
                      );
                    }
                  } catch (e) {
                    print("$e");
                  }
                }) > const Text("Unlock"),
                closeButton,
              ],
            );
          },
        );
      },
    );
  };
}