import 'package:bird/signal_wave.dart';
import 'package:bird_flutter/bird_flutter.dart';
import 'package:flutter/material.dart';

Widget signalInputWidget({
  @required Signal<String> signal,
  @required Wave<String> label,
  Wave<String> error,
  int lines = 1,
  bool isConfidential = false,
}) {
  return $$ >> (context) {
    final value = $(() => signal.wave);
    final _label = $(() => label);
    final _error = $(() => error ?? just(null));
    final controller = useTextControllerFromText(value);

    return TextFormField(
      decoration: InputDecoration(
        labelText: _label,
        border: const OutlineInputBorder(),
        errorText: _error,
      ),
      obscureText: isConfidential,
      onChanged: signal.add,
      maxLines: lines,
      controller: controller,
    );
  };
}

TextEditingController useTextControllerFromText(String txt) {
  final textController = useDispose(() {
    return TextEditingController(text: txt);
  }, (TextEditingController a) => a.dispose());

  useMemoized(() {
    final selection = textController.selection;
    textController.text = txt;
    try {
      textController.selection = selection;
    } catch (e) {
      ///
    }
  }, [txt]);

  return textController;
}