import 'dart:typed_data';

import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:pointycastle/export.dart';

class PointycastleCryptographyHashes implements CryptographyHashes {
  const PointycastleCryptographyHashes();

  @override
  CryptographyHash keccak224() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA3Digest(224).process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash keccak256() {
    return _AnonymousCryptographyHash((Uint8List intList) {
      if (intList == null) return null;
      return SHA3Digest(256).process(intList);
    });
  }

  @override
  CryptographyHash keccak384() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA3Digest(384).process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash keccak512() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA3Digest(512).process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash md2() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return MD2Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash md4() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return MD4Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash md5() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return MD5Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash ripemd128() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return RIPEMD128Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash ripemd160() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return RIPEMD160Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash ripemd256() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return RIPEMD256Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash ripemd320() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return RIPEMD320Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha1() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA1Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha224() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA224Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha256() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA256Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha384() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA384Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha512() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA512Digest().process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash sha512t(int digestSize) {
    assert(digestSize != null);
    assert(CryptographyHashes.allowedSha512tDigestSizes().contains(digestSize));
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return SHA512tDigest(digestSize).process(Uint8List.fromList(intList));
    });
  }

  @override
  CryptographyHash tiger() {
    return _AnonymousCryptographyHash((List<int> intList) {
      if (intList == null) return null;
      return TigerDigest().process(Uint8List.fromList(intList));
    });
  }
}

class _AnonymousCryptographyHash extends CryptographyHash {
  final List<int> Function(Uint8List) convert;

  const _AnonymousCryptographyHash(this.convert);

  @override
  List<int> digestRaw(Uint8List intList) => convert(intList);
}
