import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("ciphers", () {
    const c = CryptographyCiphers.dart;

    group("AES_CBC", () {
      test("1", () {
        testCipher(
          c.aes_cbc(),
          CryptoParametersPWAndIV.create(
              FocusedCodec.hex("2b7e151628aed2a6abf7158809cf4f3c").list,
              FocusedCodec.hex("000102030405060708090a0b0c0d0e0f").list),
          FocusedCodec.hex(
              "6bc1bee22e409f96e93d7e117393172a6bc1bee22e409f96e93d7e117393172a"),
          FocusedCodec.hex(
              "7649abac8119b246cee98e9b12e9197d4cbbc858756b358125529e9698a38f44"),
        );
      });
    });

    group("AES_CTR", () {
      test("1", () {
        testCipher(
          c.aes_ctr(),
          CryptoParametersPWAndIV.create(
              FocusedCodec.hex("1e85c76114322482faa77459a67506d8").list,
              FocusedCodec.hex("0aea0ea70c06c8f5c1e0b39da6df9119").list),
          FocusedCodec.hex(
              "670786ed00b108f2b74bdc0edaac28aec5d291919c270cf4d768da42e706db5b"),
          FocusedCodec.hex(
              "678c3b751515990f27eccb95d778414b1f6d9aa603ce060d28d2b4483bcb030f"),
        );
      });
    });

    group("AES_ECB", () {
      test("1", () {
        testCipher(
          c.aes_ecb(),
          CryptoParametersPW.create(
              FocusedCodec.hex("2b7e151628aed2a6abf7158809cf4f3c").list),
          FocusedCodec.hex(
              "6bc1bee22e409f96e93d7e117393172a6bc1bee22e409f96e93d7e117393172a"),
          FocusedCodec.hex(
              "3ad77bb40d7a3660a89ecaf32466ef973ad77bb40d7a3660a89ecaf32466ef97"),
        );
      });
    });
  });
}

void testCipher<T extends CryptoParameter>(CryptoCipher<T> cipher, T params,
    FocusedCodec plain, FocusedCodec encrypted) {
  final _encrypted = cipher.encrypt(plain.rawListOfInts, params);
  expect(_encrypted, encrypted.rawListOfInts);

  final decrypted = cipher.decrypt(encrypted.rawListOfInts, params);
  expect(decrypted, plain.rawListOfInts);
}
