import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("hashes", () {
    final message =
        FocusedCodec.utf8("The quick brown fox jumps over the lazy dog");

    const hashes = CryptographyHashes.dart;

    test("Keccak224", () {
      final hash = hashes.keccak224();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "f71837502ba8e10837bdd8d365adb85591895602fc552b48b7390abd");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "310aee6b30c47350576ac2873fa89fd190cdc488442f3ef654cf23fe");
    });

    test("Keccak256", () {
      final hash = hashes.keccak256();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "c5d2460186f7233c927e7db2dcc703c0e500b653ca82273b7bfad8045d85a470");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "4d741b6f1eb29cb2a9b9911c82f56fa8d73b04959d3d9d222895df6c0b28aa15");
    });

    test("Keccak384", () {
      final hash = hashes.keccak384();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "2c23146a63a29acf99e73b88f8c24eaa7dc60aa771780ccc006afbfa8fe2479b2dd2b21362337441ac12b515911957ff");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "283990fa9d5fb731d786c5bbee94ea4db4910f18c62c03d173fc0a5e494422e8a0b3da7574dae7fa0baf005e504063b3");
    });

    test("Keccak512", () {
      final hash = hashes.keccak512();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "0eab42de4c3ceb9235fc91acffe746b29c29a8c366b7c60e4e67c466f36a4304c00fa9caf9d87976ba469bcbe06713b435f091ef2769fb160cdab33d3670680e");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "d135bb84d0439dbac432247ee573a23ea7d3c9deb2a968eb31d47c4fb45f1ef4422d6c531b5b9bd6f449ebcc449ea94d0a8f05f62130fda612da53c79659f609");
    });

    test("MD2", () {
      final hash = hashes.md2();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "8350e5a3e24c153df2275c9f80692773");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "03d85a0d629d2c442e987525319fc471");
    });

    test("MD4", () {
      final hash = hashes.md4();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "31d6cfe0d16ae931b73c59d7e0c089c0");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "1bee69a46ba811185c194762abaeae90");
    });

    test("MD5", () {
      final hash = hashes.md5();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "d41d8cd98f00b204e9800998ecf8427e");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "9e107d9d372bb6826bd81d3542a419d6");
    });

    test("RIPEMD128", () {
      final hash = hashes.ripemd128();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "cdf26213a150dc3ecb610f18f6b38b46");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "3fa9b57f053c053fbe2735b2380db596");
    });

    test("RIPEMD160", () {
      final hash = hashes.ripemd160();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "9c1185a5c5e9fc54612808977ee8f548b2258d31");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "37f332f68db77bd9d7edd4969571ad671cf9dd3b");
    });

    test("RIPEMD256", () {
      final hash = hashes.ripemd256();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "02ba4c4e5f8ecd1877fc52d64d30e37a2d9774fb1e5d026380ae0168e3c5522d");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "c3b0c2f764ac6d576a6c430fb61a6f2255b4fa833e094b1ba8c1e29b6353036f");
    });

    test("RIPEMD320", () {
      final hash = hashes.ripemd320();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "22d65d5661536cdc75c1fdf5c6de7b41b9f27325ebc61e8557177d705a0ec880151c3a32a00899b8");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "e7660e67549435c62141e51c9ab1dcc3b1ee9f65c0b3e561ae8f58c5dba3d21997781cd1cc6fbc34");
    });

    test("SHA1", () {
      final hash = hashes.sha1();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "da39a3ee5e6b4b0d3255bfef95601890afd80709");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "2fd4e1c67a2d28fced849ee1bb76e7391b93eb12");
    });

    test("SHA224", () {
      final hash = hashes.sha224();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "730e109bd7a8a32b1cb9d9a09aa2325d2430587ddbc0c38bad911525");
    });

    test("SHA256", () {
      final hash = hashes.sha256();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "d7a8fbb307d7809469ca9abcb0082e4f8d5651e46d3cdb762d02d0bf37c9e592");
    });

    test("SHA384", () {
      final hash = hashes.sha384();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "ca737f1014a48f4c0b6dd43cb177b0afd9e5169367544c494011e3317dbf9a509cb1e5dc1e85a941bbee3d7f2afbc9b1");
    });

    test("SHA512", () {
      final hash = hashes.sha512();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "07e547d9586f6a73f73fbac0435ed76951218fb7d0c8d788a309d785436bbb642e93a252a954f23912547d1e8a3b5ed6e1bfd7097821233fa0538f3db854fee6");
    });

    test("SHA512t", () {
      _nullTest(hashes.sha512t(28));

      expect(() => hashes.sha512t(null),
          throwsA(const TypeMatcher<AssertionError>()));
      expect(() => hashes.sha512t(123123),
          throwsA(const TypeMatcher<AssertionError>()));
      expect(
          digestData(hashes.sha512t(28), FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "6ed0dd02806fa89e25de060c19d3ac86cabb87d6a0ddd05c333b84f4");
      expect(
          digestData(hashes.sha512t(28), message)
              .printToTypeString(const CryptoEncodingHex()),
          "944cd2847fb54558d4775db0485a50003111c8e5daa63fe722c6aa37");
      expect(
          digestData(hashes.sha512t(32), FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "c672b8d1ef56ed28ab87c3622c5114069bdd3ad7b8f9737498d0c01ecef0967a");
      expect(
          digestData(hashes.sha512t(32), message)
              .printToTypeString(const CryptoEncodingHex()),
          "dd9d67b371519c339ed8dbd25af90e976a1eeefd4ad3d889005e532fc5bef04d");

      expect(CryptographyHashes.allowedSha512tDigestSizes().isNotEmpty, true);
    });

    test("Tiger", () {
      final hash = hashes.tiger();
      _nullTest(hash);
      expect(
          digestData(hash, FocusedCodec.hex(""))
              .printToTypeString(const CryptoEncodingHex()),
          "3293ac630c13f0245f92bbb1766e16167a4e58492dde73f3");
      expect(
          digestData(hash, message)
              .printToTypeString(const CryptoEncodingHex()),
          "6d12a41e72e644f017b6f0e2f7b44c6285f06dd5d2c5b075");
    });
  });
}

void _nullTest(CryptographyHash hash) {
  expect(hash.digestRaw(null), null);
}

FocusedCodec digestData(SimpleDigest d, FocusedCodec data) =>
    FocusedCodec.raw(const CryptoEncodingByteList(), d.digestRaw(data.list));

