import 'dart:convert';

import 'package:bird_cryptography/bird_cryptography.dart';
import 'package:test/test.dart';

void main() {
  group("$CryptoCipher", () {
    /// Nothing to test
    final _TestCryptoCipher _ = _TestCryptoCipher();

    group("$CryptoParameter", () {
      /// Nothing to test
    });

    group("$CryptoParametersPW", () {
      test("not null", () {
        // ignore: prefer_const_constructors
        expect(() => CryptoParametersPW.create(null),
            throwsA(const TypeMatcher<AssertionError>()));
      });
    });

    group("$CryptoParametersPWAndIV", () {
      test("not null", () {
        expect(
            () => CryptoParametersPWAndIV.create(
                null, FocusedCodec.utf8("-").list),
            throwsA(const TypeMatcher<AssertionError>()));
        expect(
            () => CryptoParametersPWAndIV.create(
                FocusedCodec.utf8("-").list, null),
            throwsA(const TypeMatcher<AssertionError>()));
      });
    });
  });

  test("$CryptographicDataEncoding", () {
    final _TestCryptographicDataEncoding data =
        _TestCryptographicDataEncoding();

    final en = data.encodedData("data");
    final de = data.decodedData(utf8.encode("data"));

    expect(en, FocusedCodec.encoded(_TestCryptographicDataEncoding(), "data"));
    expect(
        de,
        FocusedCodec.raw(
            _TestCryptographicDataEncoding(), utf8.encode("data")));
  });

  group("$CryptographyHash", () {
    /// Nothing to test
    final _TestCryptographyHash _ = _TestCryptographyHash();
  });

  group("$HMACCrypto", () {
    /// Nothing to test
    final _TestHMACCrypto _ = _TestHMACCrypto();
  });

  group("$KDFCrypto", () {
    /// Nothing to test
    final _TestDerivatorCrypto _ = _TestDerivatorCrypto();
  });

  group("$SimpleDigest", () {
    /// Nothing to test
    final SimpleDigest _ = _TestSimpleDigest();
  });

  group("$SimplePWDigest", () {
    /// Nothing to test
    final SimplePWDigest _ = _TestSimplePWDigest();
  });
}

class _TestSimpleDigest extends SimpleDigest {
  @override
  List<int> digestRaw(List<int> intList) => intList;
}

class _TestSimplePWDigest extends SimplePWDigest {
  @override
  List<int> digestRaw(List<int> a, List<int> pw) => List.from(a)..addAll(pw);
}

class _TestCryptoCipher extends CryptoCipher {
  @override
  List<int> decrypt(List<int> data, CryptoParameter param) => null;

  @override
  List<int> encrypt(List<int> data, CryptoParameter param) => null;
}

class _TestDerivatorCrypto extends KDFCrypto {
  @override
  List<int> digestRaw(List<int> intList, List<int> pw) => null;
}

class _TestHMACCrypto extends HMACCrypto {
  @override
  List<int> digestRaw(List<int> intList, List<int> pw) => null;
}

class _TestCryptographyHash extends CryptographyHash {
  @override
  List<int> digestRaw(List<int> intList) => null;
}

class AnonymousConverter<S, T> extends Converter<S, T> {
  final T Function(S) _convert;

  const AnonymousConverter(this._convert);

  @override
  T convert(S input) => _convert(input);
}

class _TestCryptographicDataEncoding extends CryptographicDataEncoding {
  @override
  Converter<String, List<int>> get decoder =>
      AnonymousConverter((raw) => utf8.encode(raw));

  @override
  Converter<List<int>, String> get encoder =>
      AnonymousConverter((raw) => utf8.decode(raw));
}
